
# -*- codeing = utf-8 -*-
# @File：hhDouban.py
# @Ver：1.5.2
# @Author：立树
# @Time：2021/07/07 02:02
# @IDE：PyCharm

"""
更新：
- 优化 getMovieDetail 调整演员数据格式
"""

import re
import execjs
from .hhAjax import hhGet
from bs4 import BeautifulSoup

class hhDouban(object):

    def __init__(self):
        pass

    # 豆瓣搜索
    def search(self,opt={}):
        # 配置
        hhOpt = {
            "search": "",
            "decrypt": ""
        }
        hhOpt.update(opt)

        # 参数判断
        if hhOpt["search"]=="" or hhOpt["decrypt"]=="":
            print("hhframe.hhDouban.search() Error - 请补全参数（search、decrypt）")
            return {}

        try:
            # 发起请求
            page = hhGet(f"https://search.douban.com/movie/subject_search?search_text={hhOpt['search']}&cat=1002")

            # 获取加密数据
            encrypt = re.search('window.__DATA__ = "([^"]+)"',page).group(1)
            # print(encrypt)

            # 提取解密 js
            with open(hhOpt["decrypt"], "r", encoding="UTF-8", errors="ignore") as f:
                decrypt_js = f.read()
                # print(decrypt_js)

            # 解密数据
            ctx = execjs.compile(decrypt_js)
            ret = ctx.call("decrypt",encrypt)
            return ret["payload"]
        except Exception as err:
            print(f"hhframe.hhDouban.search() Error - {err}")
            return {}

    # 豆瓣明星列表抓取
    def getStarList(self,name=""):
        # 参数判断
        if name == "":
            print("hhframe.hhDouban.getStarList() Error - 请补全参数（name）")
            return []

        try:
            # 发起请求
            page = hhGet(f"https://movie.douban.com/celebrities/search?search_text={name}&cat=1002")
            html = BeautifulSoup(page,"html.parser")
            list = html.select("div.article .result")
            stars = []
            for item in list:
                star = {
                    "name": item.select(".content h3 a")[0].text,
                    "poster": item.select(".pic img")[0].attrs["src"],
                    "url": item.select(".content h3 a")[0].attrs["href"]
                }
                infos = item.select(".content > ul li")
                star["jobs"] = infos[0].text.strip().split(" / ")
                star["id"] = re.findall("\d+",star["url"])[0]
                if len(infos) == 3:
                    dates = infos[1].text.strip().split(" 至 ")
                    birth = dates[0]
                    death = dates[1] if len(dates) == 2 else ""
                    works = infos[2].text.replace("作品:", "").strip().split(" / ")
                    star["birth"] = birth
                    star["death"] = death
                    star["works"] = works
                if len(infos) == 2:
                    works = infos[1].text.replace("作品:", "").strip().split(" / ")
                    star["works"] = works
                stars.append(star)

            return stars
        except Exception as err:
            print(f"hhframe.hhDouban.getStarList() Error - {err}")
            return []

    # 豆瓣明星详情抓取
    def getStarDetail(self,id=""):
        # 参数判断
        if id == "":
            print("hhframe.hhDouban.getStarDetail() Error - 请补全参数（id）")
            return {}

        try:
            # 发起请求
            page = hhGet(f"https://movie.douban.com/celebrity/{id}/")

            # 调试
            # import os
            # from src.hhframe import hhUrl,hhOs
            # file = "./demo_douban_star_detail.html"
            # if not os.path.exists(file):
            #     hhUrl.hhDownload(f"https://movie.douban.com/celebrity/{id}/",file)
            # page = hhOs.hhOpenFile(file,"r")

            html = BeautifulSoup(page,"html.parser")

            star = {
                "id": id,
                "poster": html.select("#headline .pic img")[0].attrs["src"]
            }
            names = html.select("#content > h1")[0].text
            name = html.select("#headline .pic img")[0].attrs["title"]
            alias = names.replace(name,"").strip()
            star["name"] = name
            star["alias"] = alias

            info = html.select("#headline .info li")
            for item in info:
                label = item.select("span")[0].text
                # print(label)
                # print(item)
                if label=="星座":
                    star["constellation"] = item.text.replace("星座:","").strip()
                if label=="出生日期":
                    dates = re.findall("((\d+)年(\d+)月(\d+)日)", item.text)
                    birth, bY, bM, bD = dates[0]
                    star["birth"] = f"{bY}-{bM}-{bD}"
                if label=="生卒日期":
                    dates = re.findall("((\d+)年(\d+)月(\d+)日)", item.text)
                    birth, bY, bM, bD = dates[0]
                    death, dY, dM, dD = dates[1]
                    star["birth"] = f"{bY}-{bM}-{bD}"
                    star["death"] = f"{dY}-{dM}-{dD}"
                if label=="出生地":
                    star["area"] = item.text.replace("出生地:","").strip()
                if label=="职业":
                    star["jobs"] = item.text.replace("职业:","").strip().split(" / ")
                if label=="更多中文名":
                    star["chinese_names"] = item.text.replace("更多中文名:","").strip().split(" / ")
                if label=="更多外文名":
                    star["other_names"] = item.text.replace("更多外文名:","").strip().split(" / ")
                if label=="imdb编号":
                    star["imdb"] = item.select("a")[0].text
                    star["imdb_url"] = item.select("a")[0].attrs["href"]
                if label=="家庭成员":
                    star["family"] = item.text.replace("家庭成员:","").strip().split(" / ")

            # 简介
            if len(html.select("#intro .all")):
                star["descp"] = html.select("#intro .all")[0].text.replace("\u3000","").replace("\r","<br>").strip()
            else:
                star["descp"] = html.select("#intro .bd")[0].text.replace("\u3000","").replace("\r","<br>").strip()

            return star
        except Exception as err:
            print(f"hhframe.hhDouban.getStarDetail() Error - {err}")
            return {}

    # 豆瓣明星详情抓取
    def getStarPhotoList(self,id="",page=1,sort="like"):
        # sort
        # - like 喜欢
        # - size 尺寸
        # - time 时间

        # 参数判断
        if id == "":
            print("hhframe.hhDouban.getStarPhoto() Error - 请补全参数（id）")
            return {}

        try:
            # 发起请求
            contt = hhGet(f"https://movie.douban.com/celebrity/{id}/photos/?type=C&start={(int(page)-1)*30}&sortby={sort}&size=a&subtype=a")

            # 调试
            # import os
            # from src.hhframe import hhUrl,hhOs
            # file = "./demo_douban_star_photo_list.html"
            # if not os.path.exists(file):
            #     hhUrl.hhDownload(f"https://movie.douban.com/celebrity/{id}/photos/?type=C&start={page-1}&sortby={sort}&size=a&subtype=a",file)
            # contt = hhOs.hhOpenFile(file,"r")

            html = BeautifulSoup(contt,"html.parser")

            star = {
                "id": id,
                "name": html.select("#content h1")[0].text.replace("的图片",""),
                "sum": 0,
                "page": page,
                "size": 30,
                "pics": [],
                "items": []
            }

            list = html.select(".article .poster-col3 li")
            for item in list:
                url = item.select(".cover a")[0].attrs["href"]
                img = item.select(".cover img")[0].attrs["src"]
                size = item.select(".prop")[0].text.strip()
                reply = item.select(".name a")[0].text if len(item.select(".name a")) else ""
                title = item.select(".name")[0].text.replace(reply, "").strip()
                star["items"].append({
                    "url": url,
                    "img": img,
                    "width": int(size.split("x")[0]),
                    "height": int(size.split("x")[1]),
                    "title": title
                })
                star["pics"].append(img)

            # 图片总数
            if len(html.select(".paginator")):
                star["sum"] = int(re.findall("(\d+)",html.select(".paginator .count")[0].text)[0])
            else:
                star["sum"] = len(list)

            return star
        except Exception as err:
            print(f"hhframe.hhDouban.getStarPhoto() Error - {err}")
            return {}

    # 豆瓣明星图片详情抓取
    # http://www.3qphp.com/python/pybase/4278.html
    def getStarPhotoDetail(self,star_id="",pic_id="",cookie=""):

        # 参数判断
        if star_id == "" or pic_id == "":
            print("hhframe.hhDouban.getStarPhotoDetail() Error - 请补全参数（star_id、pic_id）")
            return {}

        try:
            # 发起请求
            # Cookie 临时获取：
            # - 登陆豆瓣
            # - 随便打开一个图片详情（比如：https://movie.douban.com/celebrity/1013782/photo/1888623099/）
            # - F12 - Network - All - 选择第一个请求（1888623099/）- Headers - Request Headers - Cookie
            page = hhGet(
                url=f"https://movie.douban.com/celebrity/{star_id}/photo/{pic_id}/",
                headers={"Cookie":cookie}
            )

            # 调试
            # import os
            # from src.hhframe import hhUrl,hhOs
            # file = "./demo_douban_star_photo_detail.html"
            # if not os.path.exists(file):
            #     hhUrl.hhDownload(
            #         url="https://movie.douban.com/celebrity/{star_id}/photo/{pic_id}/",
            #         file=file,
            #         headers={"Cookie":cookie}
            #     )
            # page = hhOs.hhOpenFile(file,"r")

            html = BeautifulSoup(page,"html.parser")

            name = html.select("#content h1")[0].text.replace("的图片","")
            size1 = html.select("#content .poster-info .pl")[0].text.replace("原图尺寸：","")
            width = size1.split("x")[0]
            height = size1.split("x")[1]
            size2 = html.select("#content .poster-info .pl")[1].text.replace("文件大小：","")
            url = html.select("#content .photo-zoom")[0].attrs["href"]

            return {
                "star_id": star_id,
                "pic_id": pic_id,
                "name": name,
                "url": url,
                "width": width,
                "height": height,
                "size": size2
            }
        except Exception as err:
            print(f"hhframe.hhDouban.getStarPhotoDetail() Error - {err}")
            return {}

    # 豆瓣电影详情抓取
    def getMovieDetail(self,id=""):
        # 参数判断
        if id == "":
            print("hhframe.hhDouban.getMovieDetail() Error - 请补全参数（id）")
            return {}

        # 返回的数据
        movie = {"id": id}

        try:
            # 发起请求
            page = hhGet(f"https://movie.douban.com/subject/{id}/")

            # 调试
            # import os
            # from src.hhframe import hhUrl,hhOs
            # file = "./demo_douban_movie_detail.html"
            # if not os.path.exists(file):
            #     hhUrl.hhDownload(f"https://movie.douban.com/subject/{id}/",file)
            # page = hhOs.hhOpenFile(file,"r")

            html = BeautifulSoup(page,"html.parser")

            names = html.select("#content > h1 > span")[0].text
            name = html.select("#mainpic > a > img")[0].attrs["alt"]
            alias = names.replace(name,"").strip()
            movie["name"] = name
            movie["alias"] = alias

            movie["poster"] = html.select("#mainpic > a > img")[0].attrs["src"]
            movie["score"] = html.select("#interest_sectl .rating_num")[0].text
            movie["rating_num"] = html.select("#interest_sectl .rating_people span")[0].text

            # movie["director"] = "、".join(list(map(lambda item:item.text,html.select("#info > span:nth-child(1) a"))))
            # movie["writer"] = "、".join(list(map(lambda item:item.text,html.select("#info > span:nth-child(3) a"))))
            # movie["type"] = "、".join(list(map(lambda item:item.text,html.select("#info > span[property='v:genre']"))))

            for item in html.select("#info")[0].text.strip().split("\n"):
                # print(item)
                if "导演" in item:
                    movie["director"] = item.split(": ")[1].replace(" / ","、")
                    continue
                if "编剧" in item:
                    movie["writer"] = item.split(": ")[1].replace(" / ","、")
                    continue
                if "主演" in item:
                    movie["actor"] = item.split(": ")[1].replace(" / ","、")
                    continue
                if "类型" in item:
                    movie["type"] = item.split(": ")[1].replace(" / ","、")
                    continue
                if "官方网站" in item:
                    movie["type"] = item.split(": ")[1]
                    continue
                if "制片国家/地区" in item:
                    movie["area"] = item.split(": ")[1].replace(" / ","、")
                    continue
                if "语言" in item:
                    movie["language"] = item.split(": ")[1].replace(" / ","、")
                    continue
                if "上映日期" in item:
                    movie["release_time"] = item.split(": ")[1].replace(" / ","、")
                    continue
                if "片长" in item:
                    movie["duration"] = item.split(": ")[1].replace(" / ","、")
                    continue
                if "又名" in item:
                    movie["alias2"] = item.split(": ")[1].replace(" / ","、")
                    continue
                if "IMDb" in item:
                    movie["imdb"] = item.split(": ")[1]
                    continue

            # 简介
            if len(html.select(".related-info .all")):
                movie["descp"] = html.select(".related-info .all")[0].text
            else:
                movie["descp"] = html.select("#link-report > span")[0].text

            movie["descp"] = movie["descp"].replace("\u3000","").replace("\r","<br>").strip()
            movie["descp"] = movie["descp"].replace("\n","<br>")
            movie["descp"] = re.sub("\s","",movie["descp"])

        except Exception as err:
            print(f"hhframe.hhDouban.getMovieDetail() Error - {err}")
            return {}

        try:
            # 发起请求
            page = hhGet(f"https://movie.douban.com/subject/{id}/celebrities")

            # 调试
            # import os
            # from src.hhframe import hhUrl,hhOs
            # file = "./demo_douban_movie_actors.html"
            # if not os.path.exists(file):
            #     hhUrl.hhDownload(f"https://movie.douban.com/subject/{id}/celebrities",file)
            # page = hhOs.hhOpenFile(file,"r")

            html = BeautifulSoup(page,"html.parser")

            actors = []
            for actor in html.select("#celebrities .list-wrapper:nth-child(2) li"):
                actors.append({
                    "name": actor.select(".info .name a.name")[0].text,
                    "role": "".join(re.findall("\([饰|配] (.*)\)", actor.select(".info .role")[0].text))
                })
            movie["actor2"] = actors

            return movie
        except Exception as err:
            print(f"hhframe.hhDouban.getMovieDetail() Error - {err}")
            return {}




# -*- codeing = utf-8 -*-
# @File：hhUrl.py
# @Ver：1.1.0
# @Author：立树
# @Time：2021/07/04 05:23
# @IDE：PyCharm

"""
更新：
- 优化 hhDownload 新增了 headers 参数
"""

import re
import urllib.request,urllib.error,urllib.parse
from .hhOs import hhCreateFile

# 下载文件
def hhDownload(url="",savefile="",headers={}):
    # 参数判断
    if url == "":
        print("hhframe.hhUrl.hhDownload() Error - 请补全参数（url）")
        return {"state":False,"msg":"参数错误"}

    # 保存的文件
    if savefile=="":
        url2 = re.sub("http[s]?://","",url)
        if url2.find("/")>-1 and url2[-1]!="/":
            savefile = url2[url2.rfind("/")+1:len(url2)]
        else:
            savefile = "tempfile"

    # 请求数据
    header = {"User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.183 Safari/537.36"}
    header.update(headers)
    request = urllib.request.Request(
        url = urllib.request.quote(url,safe=";/?:@&=+$,",encoding="utf-8"),
        method = "GET",
        headers = header
    )
    # 返回数据
    hhRet = {
        "state": None,
        "url": url,
        "savefile": savefile,
        "code": -1,
        "msg": ""
    }
    # 发起请求
    try:
        response = urllib.request.urlopen(request,timeout=60)
        hhCreateFile(savefile,"wb",response.read())
        hhRet["state"] = True
        hhRet["code"] = 200
        hhRet["msg"] = "下载成功"
    except (urllib.error.URLError,IOError) as err:
        hhRet["state"] = False
        if hasattr(err,"code"):
            hhRet["code"] = err.code
        if hasattr(err,"reason"):
            hhRet["msg"] = err.reason
    finally:
        print(" hhframe.hhUrl.hhDownload() ".center(100,"="))
        print("url".ljust(10),hhRet["url"])
        print("savefile".ljust(10),hhRet["savefile"])
        print("result".ljust(10),hhRet["code"],hhRet["msg"])
        print("="*100)
        return hhRet

# url 解析
def hhParse(url=""):
    # 解析结果
    hhRet = {
        "url": url,
        "domain": "",
        "host": "",
        "ip": "",
        "protocol": "",
        "path": "",
        "file": "",
        "filename": "",
        "ext": "",
        "query": "",
        "hash": ""
    }
    # 参数判断
    if url == "":
        return hhRet
    # 协议
    if url.find("://"):
        protocal = url[0:url.find("://")+3]
        hhRet["protocol"] = protocal
        url = url.replace(protocal, "")
    # 主机
    ret = re.findall("^(([a-z\d]([a-z\d-]{0,57}[a-z\d])?.)+(com|net|cn|org|me|xyz|top|tech|gov|edu|ink|int|pub|mil|biz|info|mobi|name|tv))",url)
    host = ret[0][0] if len(ret) else ""
    hhRet["host"] = host
    url = url.replace(host, "")
    # IP
    ret = re.findall("^(((25[0-5])|(2[0-4]\d)|(1\d\d)|([1-9]\d)|\d)(\.((25[0-5])|(2[0-4]\d)|(1\d\d)|([1-9]\d)|\d)){3})",url)
    ip = ret[0][0] if len(ret) else ""
    hhRet["ip"] = ip
    url = url.replace(ip, "")
    # 端口
    ret = re.findall("^:(\d+)",url)
    port = ret[0] if len(ret) else ""
    hhRet["port"] = port
    url = url.replace(f":{port}", "")
    # 域名
    hhRet["host"] = hhRet["ip"] if not hhRet["host"] else hhRet["host"]
    hhRet["domain"] = hhRet["protocol"]+hhRet["host"]+(f":{hhRet['port']}" if hhRet["port"] else "")
    # 锚点
    if url.find("#") > -1:
        hash = url[url.find("#"):len(url)]
        hhRet["hash"] = hash.replace("#", "")
        url = url.replace(hash, "")
    # 查询字符串
    if url.find("?") > -1:
        query = url[url.find("?"):len(url)]
        hhRet["query"] = query.replace("?", "")
        url = url.replace(query, "")
    # 文件
    if url.find(".")>-1:
        file = url[url.rfind("/")+1:len(url)]
        hhRet["file"] = file
        hhRet["path"] = url.replace(hhRet["file"],"")
        hhRet["filename"] = file[0:file.rfind(".")]
        hhRet["ext"] = file.replace(hhRet["filename"]+".","")
    else:
        hhRet["path"] = url

    return hhRet




# -*- codeing = utf-8 -*-
# @File：hhOs.py
# @Ver：1.2.0
# @Author：立树
# @Time：2021/07/04 18:00
# @IDE：PyCharm

"""
更新：
- 新增 hhOpenFile 打开文件功能
"""

import os

# 打开文件（文本类型的文件）
def hhOpenFile(file="",mode="r",encoding="utf-8"):
    # 参数判断
    if file == "":
        print("hhframe.hhOs.hhOpenFile() Error - 请补全参数（file）")
        return ""

    # 路径判断
    if not os.path.exists(file):
        print(f"hhframe.hhOs.hhOpenFile() Error - '{file}'文件路径不存在")
        return ""

    try:
        with open(file,mode,encoding=encoding) as f:
            contt = f.read()
            # print(contt)
            return contt
    except Exception as err:
        print(f"hhframe.hhOs.hhOpenFile() Error - {str(err)}")
        return ""

# 创建文件
def hhCreateFile(path="",mode="w",content="",encoding=None):
    # 参数判断
    if path == "" or content == "":
        print("hhframe.hhOs.hhCreateFile() Error - 请补全参数（path、content）")
        return {"state": False, "msg": "缺少参数"}

    # 路径检测
    if path.find("/")>-1:
        if path[-1]!="/":
            # 匹配内容：
            # - [./][dir/].filename
            # - [./][dir/]filename
            # - [./][dir/]filename.ext
            dir = path[0:path.rfind("/")+1]
            if not os.path.exists(dir):
               os.makedirs(dir)
        else:
            # 匹配内容：
            # - ./
            # - ./dir/
            # - ./dir/dir/
            # - ../dir/
            # - dir/dir/
            print(f"hhframe.hhOs.hhCreateFile() Error - 参数不合法（'{path}'不是有效文件路径）")
            return {"state": False, "msg": f"参数不合法（'{path}'不是有效文件路径）"}
    else:
        # 匹配内容：
        # - .filename
        # - filename
        # - filename.ext
        pass

    try:
        with open(path,mode,encoding=encoding) as f:
            f.write(content)
        return {"state": True, "msg": "保存成功"}
    except IOError as err:
        return {"state": False, "msg": str(err)}

# 删除文件、文件夹
def hhDelete(path=""):
    # 参数判断
    if path == "":
        print("hhframe.hhOs.hhDelete() Error - 请补全参数（path）")
        return {"state": False, "msg": "缺少参数"}

    # 路径判断
    if not os.path.exists(path):
        print(f"hhframe.hhOs.hhDelete() Error - 路径不存在（{path}）")
        return {"state": False, "msg": f"路径不存在（{path}）"}

    # 屏蔽风险操作
    if path=="." or path=="./" or path[0]=="/" or path.find("..")>-1:
        print(f"hhframe.hhOs.hhDelete() Error - 路径无法删除（{path}）")
        return {"state": False, "msg": f"路径无法删除（{path}）"}

    # 删除
    try:
        if os.path.isfile(path):
            # 删除文件
            os.remove(path)
            return {"state": True, "msg": "文件删除成功"}
        else:
            # 删除当前文件夹下的所有文件、子文件夹
            for root, dirs, files in os.walk(path, topdown=False):
                for name in files:
                    file = os.path.join(root, name).replace("\\", "/")
                    # print("file - ", file)
                    if os.path.exists(file):
                        os.remove(file)
                for name in dirs:
                    dir = os.path.join(root, name).replace("\\", "/")
                    # print("folder - ", dir)
                    if os.path.exists(dir):
                        os.removedirs(dir)
            # 删除当前文件夹
            os.removedirs(path)
            return {"state": True, "msg": "文件夹删除成功"}
    except IOError as err:
        return {"state": False, "msg": str(err)}
